using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
//using System.Runtime;
using System.IO;
//using System.Runtime.ExceptionServices;
using System.Security.Permissions;
using UnityEngine;
using TMPro;
//using Steamworks;
//compiler disallows users to reference steamworks
//using UnityEngine.UIElements;
using UnityEngine.Events;
namespace stickpart {
	public static class ContraptionUtilities 
	{
		public static string UserReadable(this bool b)
		{
			if(b){return "ON";}else{return "OFF";}
		}
		public static DistanceJoint2D CreateWire<T>(this GameObject rb,Rigidbody2D otherbody,Vector2[] Anchors,float distance = 2f) where T : DistanceJointWireBehaviour
		{
			DistanceJoint2D joint = rb.AddComponent<DistanceJoint2D>();
                joint.maxDistanceOnly = true;
                joint.enableCollision = true;
                joint.distance = distance;
                joint.autoConfigureDistance = false;
                joint.breakForce = Mathf.Infinity;
                joint.breakTorque = Mathf.Infinity;
                joint.connectedBody = otherbody;
				joint.anchor = new Vector2(-0.1f, 0f);
                joint.connectedAnchor = new Vector2(0.1f,0f);
				T energyWireBehaviour = joint.gameObject.AddComponent<T>();
            energyWireBehaviour.WireColor = new Color32(0,0,0,255);
            energyWireBehaviour.WireMaterial = Resources.Load<Material>("Materials/Wire");
            energyWireBehaviour.WireWidth = 0.025f;
            energyWireBehaviour.typedJoint = joint;
			return joint;
		}
		  public static void HeatDisable(GameObject obj, bool spriteOnly = false,bool state = false,bool dontremovemetal = false) {
			  if (!spriteOnly) {
				PhysicalBehaviour[] s = obj.GetComponentsInChildren < PhysicalBehaviour > (true);
				if (s != null) {
				  foreach(PhysicalBehaviour sr in s) {
					sr.SimulateTemperature = state;
				  }
				}
			  }
			  if(dontremovemetal) {return;}
			  GlowingHotMetalBehaviour[] r = obj.GetComponentsInChildren < GlowingHotMetalBehaviour > ();
			  if (r != null) {
				foreach(GlowingHotMetalBehaviour beh in r) {
				  beh.gameObject.GetComponent < SpriteRenderer > ().sharedMaterial = ModAPI.FindMaterial("Sprites-Default");
				  UnityEngine.Object.Destroy(beh);
				}
			  }
		}
		public static async void mp3Download(string path,Action<AudioClip> workaround,string cachedesignator = "unnamed")
		{
			if(String.IsNullOrWhiteSpace(path))
		  {return;}		  
		string currentdir = Directory.GetCurrentDirectory();
		string cached = currentdir+@"\"+cachedesignator+"_cache";
		string filename = Path.GetFileName(path);
		Debug.Log(filename);
		if(!Directory.Exists(cached))
		{
			Directory.CreateDirectory(cached);
		}
		string filepath = cached+@"\"+filename;
		Debug.Log(filepath);
		if(!File.Exists(filepath))
		{
		byte[] content = await Utils.HttpDownload(path);
		File.WriteAllBytes(filepath,content);
		}
		//var sex = await Utils.FileToAudioClip(filepath);
		workaround(Utils.FileToAudioClip(filepath));
		}
		public static string FixedModPath(string path)
		{
			return path.Replace(@"\mod.json","")+@"\";
		}
		public static Texture2D LoadTexture(string filename,FilterMode mode = FilterMode.Point)
		{
			string prt = ContraptionUtilities.ModPath();
			return Utils.LoadTexture(FixedModPath(prt)+filename,mode);
		}
		public static string ModPath(bool log = false) {
  string x = "";
  foreach(ModMetaData data in ModLoader.LoadedMods) {
    if (data.EntryPoint == "stickpart.Mod") // if its this mod
    {
      if (!ModLoader.ModScripts[data].RelativePath.StartsWith(Directory.GetCurrentDirectory().Substring(0, 2))) {
        x = Directory.GetCurrentDirectory() + @"\"+ModLoader.ModScripts[data].RelativePath;
        //if(log){Debug.Log("Reverting to method 1");
      } else {
        //Debug.Log("Anomaly, Reverting to method 2");
        x = ModLoader.ModScripts[data].RelativePath;
      }
    }

  }
  return x;
}
public static void CreateGlowObject(this GameObject obj,Sprite sprite,Vector3 offset,Vector3 scale,Color32? tint = null,bool glow = true,string extraname = "")
	   {
		   if(!obj.transform.Find("glow"+extraname))
		   {
			   //Debug.Log("Creating glow");
		   var childObject = new GameObject("glow"+extraname);
                     childObject.transform.SetParent(obj.transform);
                     childObject.transform.position = obj.transform.position;
                     childObject.transform.localPosition = new Vector3(offset.x,offset.y);
                     childObject.transform.localScale = scale;
					 childObject.transform.localRotation = Quaternion.Euler(0f,0f,0f);
                     var childSprite = childObject.AddComponent < SpriteRenderer > ();
                     childSprite.sprite = sprite;
					 //childObject.layer = 9;
					 SpriteRenderer objrenderer = obj.GetComponent<SpriteRenderer>();
					 childSprite.sortingLayerName = objrenderer.sortingLayerName;
					 childSprite.sortingOrder = objrenderer.sortingOrder + 2;
                     //childSprite.sortingLayerName = "Top";
					 childSprite.color = tint ?? new Color32(255,255,255,255);
					 if(glow)
					 {
                     childSprite.sharedMaterial = ModAPI.FindMaterial("VeryBright");
					 }
                     childObject.SetActive(false);
		   }
	   }
	}
   public class Mod {
	   public static void ComponentStrip(GameObject obj)
	   {
		   //very much adapted to the Logic Gates, probs dont use?
		   foreach (var comp in obj.GetComponents<Component>())
 {
     if (!(comp is Joint2D) &&!(comp is FixedJoint2D) &&!(comp is HeightSensor)&&!(comp is WireBehaviour) &&!(comp is Transform) && !(comp is Collider2D) && !(comp is FreezeBehaviour) && !(comp is LogicGateBase)&& !(comp is LogicNode)&& !(comp is Rigidbody2D) && !(comp is SpriteRenderer) && !(comp is PhysicalBehaviour) && !(comp is DistanceJoint2D) && !(comp is SerialisableIdentity)&& !(comp is LineRenderer)&& !(comp is SmoothLineControllerBehaviour) )
     {
		 //Debug.Log("Destroying: " + comp.ToString());
         UnityEngine.Object.Destroy(comp);
     }
 }
	   }
	   public static void ChangeWeight(GameObject obj,float weight)
	   {
		   obj.GetComponent<Rigidbody2D>().mass = weight;
	   }
	  
	   //public class secksurdad : MonoBehaviour {}
	   public static void LogicGateSetup < T > (GameObject Instance,Sprite glow,bool deletetext = true,Quaternion[] Rotations = null) where T: LogicGateBase {
		   Quaternion[] RRotations = Rotations;
		   if(Rotations == null)
		   {
			   RRotations = new Quaternion[]{Quaternion.Euler(0f,0f,0f),Quaternion.Euler(0f,0f,0f)};
		   }
		   // COMPONENT STRIP
		   GameObject MainBody = Instance.transform.Find("defib body").gameObject;
		   ChangeWeight(MainBody,2f);
		   GameObject Slot1_b = Instance.transform.Find("defib pad").gameObject;
		   if(deletetext)
		   {
		   UnityEngine.Object.Destroy(MainBody.transform.Find("text").gameObject);
		   }
		   GameObject Slot2_b = Instance.transform.Find("defib pad 2").gameObject;
		   ComponentStrip(MainBody);
		   ComponentStrip(Slot1_b);
		   ComponentStrip(Slot2_b);
		   //Debug.Log(Slot2_b.GetComponent<DistanceJoint2D>().connectedBody.gameObject.name);
		   
		   //Slot1_b.name = "slot1";
		   //Slot2_b.name = "slot2";
		   //Slot INIT
                     var Slot1 = Slot1_b;
					 //Slot1.FixColliders();
			        Slot1.transform.SetParent(MainBody.transform);
			        Slot1.transform.localPosition = new Vector3(-0.2f, -0.4425f);
					Slot1.transform.localRotation = RRotations[0];
					var Slot2 = Slot2_b;
			        Slot2.transform.SetParent(MainBody.transform);
			        Slot2.transform.localPosition = new Vector3(0.2f, -0.4425f);
					Slot2.transform.localRotation = RRotations[1];
					Sprite glownode = ModAPI.LoadSprite("logicgate/nodeglow.png");
					GameObject[] Slots = new GameObject[]{Slot1,Slot2};
					foreach(GameObject dasdassad in Slots)
					{
						//UnityEngine.Object.Destroy(dasdassad.GetComponent<LineRenderer>());
					foreach (DistanceJoint2D joint in dasdassad.GetComponents(typeof(DistanceJoint2D)))
					{
						if(joint.connectedBody.gameObject.name == "defib body")
						{
							UnityEngine.Object.Destroy(dasdassad.GetComponent<LineRenderer>());
							UnityEngine.Object.Destroy(joint);
						}			
					}
					}
					//END SLOT WELD,BEGIN LOGIC SETUP
					LogicGateBase LogicGate;
					MainBody.CreateGlowObject(glow,new Vector3(0f,0f),new Vector3(1f,1f));
					if(MainBody.GetComponent<T>() == null)
					{
					 LogicGate = (LogicGateBase)MainBody.AddComponent(typeof (T));
					}
					else {
						LogicGate = (LogicGateBase)MainBody.GetComponent(typeof (T));
					}
					int slot = 1;
					foreach(GameObject obj in Slots)
					{
						obj.transform.SetParent(Instance.transform);
						obj.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("logicgate/LogicNode.png");
					    obj.FixColliders();
						PhysicalBehaviour rg = obj.GetComponent<PhysicalBehaviour>() as PhysicalBehaviour;
						rg.MakeWeightless();
						obj.GetComponent<SpriteRenderer>().sortingLayerName = "Top";
						obj.CreateGlowObject(glownode,new Vector3(0f,0f),new Vector3(1f,1f),new Color32(76,255,0,255));
						FixedJoint2D joint = MainBody.AddComponent<FixedJoint2D>();
					joint.connectedBody = obj.GetComponent<Rigidbody2D>();
					joint.enableCollision = false;
						//rg.Selectable = false;
						//Make sure offsets are correct before making unselectable.
					if(obj.GetComponent<LogicNode>() == null)
					{
					 LogicNode logical = obj.AddComponent(typeof (LogicNode)) as LogicNode;
					 logical.slot = slot;
					 logical.Logic = (LogicGateBase)LogicGate;
					}slot++;}
	   }
		public static void OnLoad()
		{
			ModAPI.RegisterInput("Autofreeze", "autofreeze-PNC_jimmyl", KeyCode.P);
			ModAPI.RegisterInput("Stretch Cables", "cablestretch-PNC_jimmyl", KeyCode.O);
			ModAPI.RegisterInput("Save selected item", "qs_save-jimmyl", KeyCode.Keypad1);
            ModAPI.RegisterInput("Load saved selected item", "qs_load-jimmyl", KeyCode.Keypad0);
		}
		[SkipSerialisation]
		public static bool Autofreeze_Enabled = false;
		[SkipSerialisation]
		public static bool Stretchy_Cables = false;
		public static List<DialogBox> DialogsToClose = new List<DialogBox>();
      public static void Main() {
		  ModAPI.OnItemSpawned += (sender, args) => {
		  GameObject obj = args.Instance;
		  //Debug.Log(obj.GetComponent<GeneratorBehaviour>().TargetCharge);
		  PhysicalBehaviour whatever = obj.GetComponent<PhysicalBehaviour>();
		  if(Autofreeze_Enabled)
		  {
			  if(whatever != null)
			  {
			  obj.GetOrAddComponent<FreezeBehaviour>();
			  }
			  else
			  {
				  foreach(PhysicalBehaviour pr in obj.GetComponentsInChildren<PhysicalBehaviour>(false))
				  {
					  pr.gameObject.GetOrAddComponent<FreezeBehaviour>();
				  }
			  }
		  }
		};
		ModAPI.OnWireCreated += (sender, wire) => {
			if (wire.GetType().BaseType.Name == "DistanceJointWireBehaviour" && Stretchy_Cables)
			{
				DistanceJointWireBehaviour actualwire = wire as DistanceJointWireBehaviour;
			    actualwire.typedJoint.enabled = false;
				StretchyCableFix savefix = actualwire.typedJoint.gameObject.AddComponent<StretchyCableFix>();
				Debug.Log(actualwire.currentDistance);
				savefix.IDs=actualwire.currentDistance;
			}
		};
		ModAPI.RegisterPower<mutepowah>(
			"Mute Power", 
			"Mutes/Unmutes clicked object.", 
			ModAPI.LoadSprite("mute.png",1,false)
		);
		ModAPI.RegisterPower<InvisibilityPower>(
			"Invisibility Power", 
			"Makes clicked object invisible.", 
			ModAPI.LoadSprite("invis.png",1,false)
		);
		ModAPI.RegisterTool<NoColPower>(
			"No-Collision Constraint", 
			"Click two items to make them not collide", 
			ModAPI.LoadSprite("nocol.png",1,false),"Phase link"
		);
		ModAPI.RegisterTool<ToggleableSpringTool>(
			"Activation Spring", 
			"Activate the base of the spring to toggle if the spring actually does anything", 
			ModAPI.LoadSprite("springplaceholder.png",1,false),"Spring"
		);
		ModAPI.Register<AutofreezeBehaviour>();ModAPI.Register<DialogCloserESC>();ModAPI.Register<QuickSelector>();
		//ModAPI.RegisterTool<DurableSteelBindingTool>("Durable Steel Binding", "A Steel Binding that's 5 times as strong.", ModAPI.LoadSprite("durablet.png",0.0285714286f,false), "Fixed cable");
		//ModAPI.RegisterTool<StrongSteelBindingTool>("Strong Steel Binding", "A Steel Binding that's 10 times as strong.", ModAPI.LoadSprite("strongt.png",0.0285714286f,false), "Fixed cable");
		  //Debug.Log(SteamUser.GetSteamID().ToString());
         ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Soap"),
                  NameOverride = "Sticky Part",
                  DescriptionOverride = "A block of really sticky activatable glue.",
                  NameToOrderByOverride = "zzzz",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("view.png"),
                  AfterSpawn = (Instance) => {
                     //soapbehaviour
                     UnityEngine.Object.Destroy(Instance.GetComponent < SoapBehaviour > ());
					 Instance.CreateGlowObject(ModAPI.LoadSprite("g_part.png"),new Vector3(0f, 0f),new Vector3(1f,1f));
					 var blg = new Sprite[]{ModAPI.LoadSprite("part.png"),ModAPI.LoadSprite("moretextures/cubestickyA.png"),ModAPI.LoadSprite("moretextures/cubestickyB.png")};
					 var dupe = ModAPI.LoadSprite("moretextures/g_p1.png");
					 var g_blg = new Sprite[]{ModAPI.LoadSprite("g_part.png"),dupe,dupe};
						SpriteChangerBehaviour Changer = Instance.GetOrAddComponent<SpriteChangerBehaviour>();
						//Debug.Log("possible error start");
						Changer.sprites = blg;
						Changer.glowsprites = g_blg;
						Changer.Init();
						Instance.GetOrAddComponent<StickyBehaviour>();
                  }
            });
			ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Rod"),
                  NameOverride = "Collision Part",
				  NameToOrderByOverride = "zzzz",
                  DescriptionOverride = "Toggles collision when activated.",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("cth.png",5f),
                  AfterSpawn = (Instance) => {
                     Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("colpart.png");
                     Instance.FixColliders();
					 Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Bottom";
					 ChangeWeight(Instance,2f);
					 ContraptionUtilities.HeatDisable(Instance,true);
					 if(Instance.GetComponent<CollisionToggler>() == null)
					 {
                     Instance.AddComponent(typeof(CollisionToggler));
					 }
                  }
            });
			ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Rod"),
                  NameOverride = "Longer Collision Part",
				  NameToOrderByOverride = "zzzz",
                  DescriptionOverride = "Toggles collision when activated. 2X long.",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("lcth.png",5f),
                  AfterSpawn = (Instance) => {
                     Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("longcolpart.png");
                     Instance.FixColliders();
					 Instance.GetComponent<SpriteRenderer>().sortingLayerName = "Bottom";
					 ChangeWeight(Instance,3f);
					 ContraptionUtilities.HeatDisable(Instance,true);
					 if(Instance.GetComponent<CollisionToggler>() == null)
					 {
                     Instance.AddComponent(typeof(CollisionToggler));
					 }
                  }
            });
			//Vector2[] pointed = {new Vector2(1.3f,-3.2f),new Vector2(22.7f, -9.1f),new Vector2(22.7f, 9.5f),new Vector2(1.3f, 3.1f),new Vector2(0.5f, 0.1f)};
			/*ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Life Detector"),
                  NameOverride = "Conical Life Detector",
                  DescriptionOverride = "poop",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("view.png"),
                  AfterSpawn = (Instance) => {
					  Vector2[] pointed = {new Vector2(1.3f,-1.7f),new Vector2(8.0f, -4.5f),new Vector2(8.0f, 4.5f),new Vector2(1.3f, 1.7f),new Vector2(0.5f, 0.1f)};
					  PolygonCollider2D whatever = Instance.AddComponent(typeof (PolygonCollider2D)) as PolygonCollider2D;
					  whatever.points = pointed;
					  whatever.isTrigger = true;
					  whatever.offset =new Vector2(-0.4f,-0.1f);
					  debugcoll coldrawer = Instance.AddComponent(typeof (debugcoll)) as debugcoll;
					  coldrawer.col = whatever;
					  DetectorBehaviour detector = Instance.GetComponent < DetectorBehaviour > () as DetectorBehaviour;
					  var childObject = new GameObject("rend");
                      childObject.transform.SetParent(Instance.transform);
					  MeshFilter filter = childObject.AddComponent(typeof (MeshFilter)) as MeshFilter;
					  filter.mesh = whatever.CreateMesh(true,true);
					  MeshRenderer mrenderer = childObject.AddComponent(typeof (MeshRenderer)) as MeshRenderer;
					  mrenderer.material = ModAPI.FindMaterial("Sprites-Default");
					  mrenderer.material.color = new Color32(0,0,220,80);
					  detector.Line.enabled = false;
                  }
            });*/
			//
			ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Rod"),
                  NameOverride = "Enemy Core",
				  NameToOrderByOverride = "zzzz",
                  DescriptionOverride = "Useful for automatic machine enemies.",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("enemyt.png"),
                  AfterSpawn = (Instance) => {
                     Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("enemy.png");
                     Instance.FixColliders();
					 Instance.name = "EnemyCore";
                     Instance.CreateGlowObject(ModAPI.LoadSprite("glowred.png"),new Vector3(-0.085f, 0f),new Vector3(1f,1f));
					 Rigidbody2D RB = Instance.GetComponent<Rigidbody2D>() as Rigidbody2D;
					 RB.useAutoMass = false;
					 RB.mass = 1f;
                     FloatEnemyAI AI = Instance.GetOrAddComponent<FloatEnemyAI>();
					 AI.DefaultLight = ModAPI.LoadSprite("glowred.png");
					 AI.BattleLight = ModAPI.LoadSprite("glowblue.png");
                  }
            });
			ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Rod"),
                  NameOverride = "Blood Sucker",
				  NameToOrderByOverride = "zzzz",
                  DescriptionOverride = "Definently illegal.",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("bloodsuckert.png"),
                  AfterSpawn = (Instance) => {
                     UnityEngine.Object.Destroy(Instance.GetComponent < SoapBehaviour > ());
                     Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("bloodsucker.png");
                     Instance.FixColliders();
                     Instance.AddComponent(typeof (BloodTransferer));
					 if(Instance.GetComponent<BloodSucker>() == null)
					{
					 Instance.AddComponent(typeof (BloodSucker));
					}
                  }
            });
			//
         ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Rod"),
                  NameOverride = "Node",
                  DescriptionOverride = "DEPRECATED PLEASE USE HIGHSPEED NODE",
                  CategoryOverride = ModAPI.FindCategory("asadsad"),
                  ThumbnailOverride = ModAPI.LoadSprite("nodethumb.png"),
                  AfterSpawn = (Instance) => {
                     Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("node.png");
                     Instance.FixColliders();
					 Instance.AddComponent(typeof (BloodTransferer));
                  }
            });
			ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Rod"),
                  NameOverride = "High-speed Node",
				  NameToOrderByOverride = "zzzz",
                  DescriptionOverride = "Cable Management. Customizable activation delay.",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("hsnt.png"),
                  AfterSpawn = (Instance) => {
                     Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("highspeednode.png");
                     Instance.FixColliders();
					 Instance.GetOrAddComponent<BloodTransferer>();
					 Instance.GetOrAddComponent<CustomizableAD>();
                  }
            });
			/*ModAPI.Register(
			  new Modification() {
			    OriginalItem = ModAPI.FindSpawnable("Rod"),
			      NameOverride = "Plug",
			      DescriptionOverride = "Plugs.",
			      CategoryOverride = ModAPI.FindCategory("Machinery"),
			      ThumbnailOverride = ModAPI.LoadSprite("plugt.png"),
			      AfterSpawn = (Instance) => {
			        var Plug2 = ModAPI.CreatePhysicalObject("plug", ModAPI.LoadSprite("plug2.png"));
					Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("plug1.png");
					Instance.name = "plug";
			        Instance.FixColliders();
					Plug2.FixColliders();
			        Plug2.transform.SetParent(Instance.transform);
			        Plug2.transform.localPosition = new Vector3(-1, 0);
					DistanceJoint2D joint = Instance.AddComponent<DistanceJoint2D>();
                joint.maxDistanceOnly = true;
                joint.enableCollision = true;
                joint.distance = 2f;
                joint.autoConfigureDistance = false;
                joint.breakForce = Mathf.Infinity;
                joint.breakTorque = Mathf.Infinity;
                joint.connectedBody = Plug2.GetComponent<Rigidbody2D>();
				joint.anchor = new Vector2(-0.1f, 0f);
                joint.connectedAnchor = new Vector2(0.1f,0f);
				EnergyWireBehaviour energyWireBehaviour = joint.gameObject.AddComponent<EnergyWireBehaviour>();
            energyWireBehaviour.WireColor = new Color32(0,0,0,255);
            energyWireBehaviour.WireMaterial = Resources.Load<Material>("Materials/Wire");
            energyWireBehaviour.WireWidth = 0.025f;
            energyWireBehaviour.typedJoint = joint;
			var phys = Instance.GetComponent<PhysicalBehaviour>();
			phys.Resizable = false;
			phys = Plug2.GetComponent<PhysicalBehaviour>();
			phys.Resizable = false;
			      }
			  });*/
			ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Rod"),
                  NameOverride = "Beeper",
				  NameToOrderByOverride = "zzzz",
                  DescriptionOverride = "Beeps. (Can also play sound from link)",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("beepert.png"),
                  AfterSpawn = (Instance) => {
                     Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("beeper.png");
                     Instance.FixColliders();
					 BeepyBehaviour beeper = Instance.GetOrAddComponent<BeepyBehaviour>();
					 AudioSource ass = Instance.AddComponent(typeof (AudioSource)) as AudioSource;
					 beeper.audioclip = ModAPI.LoadSound("beep2.mp3");
					 beeper.audioclipB = ModAPI.LoadSound("beep.mp3");
					 beeper.audio = ass;
                  }
            });
			ModAPI.Register(
			  new Modification() {
			    OriginalItem = ModAPI.FindSpawnable("Rod"),
			      NameOverride = "Minigame Teleporter",
				  NameToOrderByOverride = "zzzz",
			      DescriptionOverride = "Teleports the player to a random endpoint with the same channel when touched.",
			      CategoryOverride = ModAPI.FindCategory("Machinery"),
			      ThumbnailOverride = ModAPI.LoadSprite("portalthumb.png"),
			      AfterSpawn = (Instance) => {
					Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("blueportal.png");
					Instance.FixColliders();
					Instance.name = "MinigameTeleporter";
					Instance.GetComponent < SpriteRenderer > ().sortingLayerName = "Bottom";
					Killblock Teleporter = Instance.GetOrAddComponent<Killblock>();
					Teleporter.BluePortal = ModAPI.LoadSprite("blueportal.png");
					Teleporter.OrangePortal = ModAPI.LoadSprite("orangeportal.png");
					Teleporter.UnactivePortal = ModAPI.LoadSprite("emptyportal.png");
			      }
			  });
			ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Rod"),
                  NameOverride = "Zero Point Battery",
				  NameToOrderByOverride = "zzzz",
                  DescriptionOverride = "poggers",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("zpbt.png"),
                  AfterSpawn = (Instance) => {
					  Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("zpb.png");
					  Instance.GetComponent<PhysicalBehaviour>().ForceNoChargeParticles = true;
                      Instance.FixColliders();
					  if(Instance.GetComponent<custombat>() == null)
					{
						Instance.AddComponent(typeof (custombat));
					}
                  }
            });
			ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Rod"),
                  NameOverride = "ColDebugLister",
                  DescriptionOverride = "comes deactivated",
                  CategoryOverride = ModAPI.FindCategory("what"),
                  ThumbnailOverride = ModAPI.LoadSprite("zpbt.png"),
                  AfterSpawn = (Instance) => {
					  Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("dbc.png");
                      Instance.FixColliders();
					  if(Instance.GetComponent<CollisionLister>() == null)
					{
						Instance.AddComponent(typeof (CollisionLister));
					}
                  }
            });
			ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Rod"),
                  NameOverride = "Floater",
				  NameToOrderByOverride = "zzzz",
                  DescriptionOverride = "Using very advanced techonology it sucks air and floats. May not look like a game reference.",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("floatert.png"),
                  AfterSpawn = (Instance) => {
					  Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("floater.png");
                      Instance.FixColliders();
					  if(Instance.GetComponent<floater>() == null)
					{
						Instance.AddComponent(typeof (floater));
					}
                  }
            });
			ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Rod"),
                  NameOverride = "Constant Activation Box",
				  NameToOrderByOverride = "zzzz",
                  DescriptionOverride = "Red wire for normal mode. Use blue wire for turbo mode.",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("acspamt.png"),
                  AfterSpawn = (Instance) => {
					  Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("acspam.png");
                      Instance.FixColliders();
					  if(Instance.GetComponent<ConstantActivator>() == null)
					{
						Instance.AddComponent(typeof (ConstantActivator));
					}
                  }
            });
			ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Rod"),
                  NameOverride = "Dead Weight",
				  NameToOrderByOverride = "zzzz",
                  DescriptionOverride = "Customizable weight.",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("weightB.png"),
                  AfterSpawn = (Instance) => {
					  Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("weight.png");
                      Instance.FixColliders();
					  if(Instance.GetComponent<customweight>() == null)
					{
						Instance.AddComponent(typeof (customweight));
					}
                  }
            });
			ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Rod"),
                  NameOverride = "Submersion Detector",
				  NameToOrderByOverride = "zzzz",
                  DescriptionOverride = "Sends out activation when half or fully submerged.",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("waterdetectort.png"),
                  AfterSpawn = (Instance) => {
					  Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("waterdetector.png");
                      Instance.FixColliders();
					  if(Instance.GetComponent<WaterSensor>() == null)
					{
						Instance.AddComponent(typeof (WaterSensor));
					}
                  }
            });
			ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Rod"),
                  NameOverride = "Light Sensor",
				  NameToOrderByOverride = "zzzz",
                  DescriptionOverride = "Sends out activation when in light.",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("LightDetectorT.png"),
                  AfterSpawn = (Instance) => {
					  Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("LightDetector.png");
                      Instance.FixColliders();
					  if(Instance.GetComponent<LightSensor>() == null)
					{
						Instance.AddComponent(typeof (LightSensor));
					}
                  }
            });
			ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Rod"),
                  NameOverride = "Angle Sensor",
				  NameToOrderByOverride = "zzzz",
                  DescriptionOverride = "Sends out activation when in the correct angle range.",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("angledt.png"),
                  AfterSpawn = (Instance) => {
					  Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("angled.png");
                      Instance.FixColliders();
					  if(Instance.GetComponent<AngleSensor>() == null)
					{
						Instance.AddComponent(typeof (AngleSensor));
					}
                  }
            });
			/*ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Laser Pointer"),
                  NameOverride = "Tagger",
                  DescriptionOverride = "Tags a part with a channel. For use with Tag Detector.",
                  CategoryOverride = ModAPI.FindCategory("Firearms"),
                  ThumbnailOverride = ModAPI.LoadSprite("taggert.png"),
                  AfterSpawn = (Instance) => {
					  Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("tagger.png");
                      Instance.FixColliders();
					  if(Instance.GetComponent<TaggerLaser>() == null)
					{
						LaserBehaviour oglaser = Instance.GetComponent < LaserBehaviour > () as LaserBehaviour;
						TaggerLaser tagger = Instance.AddComponent(typeof (TaggerLaser)) as TaggerLaser;
						AudioSource ass = Instance.AddComponent(typeof (AudioSource)) as AudioSource;
						tagger.audioclip = ModAPI.LoadSound("beep.mp3");
						tagger.audio = ass;
						tagger.barrelPosition = new Vector3(0.2f, 0.055f);
						tagger.barrelDirection = oglaser.barrelDirection;
						tagger.PhysicalBehaviour = oglaser.PhysicalBehaviour;
						tagger.LaserColourGradient = oglaser.LaserColourGradient;
						tagger.LayersToHit = oglaser.LayersToHit;
						tagger.LineRenderer = oglaser.LineRenderer;
						tagger.Activated = true;
						UnityEngine.Object.Destroy(oglaser);
					}
                  }
            });
			ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Laser Pointer"),
                  NameOverride = "Tag Checker",
                  DescriptionOverride = "Checks for tags in anything that passes the laser.",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("checkt.png"),
                  AfterSpawn = (Instance) => {
					  Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("tagchecker.png");
                      Instance.FixColliders();
					  if(Instance.GetComponent<TagChecker>() == null)
					{
						LaserBehaviour oglaser = Instance.GetComponent < LaserBehaviour > () as LaserBehaviour;
						TagChecker tagger = Instance.AddComponent(typeof (TagChecker)) as TagChecker;
						tagger.barrelPosition = new Vector3(0f, 0f);
						tagger.barrelDirection = oglaser.barrelDirection;
						tagger.PhysicalBehaviour = oglaser.PhysicalBehaviour;
						tagger.LaserColourGradient = oglaser.LaserColourGradient;
						tagger.LayersToHit = oglaser.LayersToHit;
						tagger.LineRenderer = oglaser.LineRenderer;
						tagger.Activated = true;
						oglaser.enabled = false;
						//UnityEngine.Object.Destroy(oglaser);
					}
					else
					{
						LaserBehaviour oglaser = Instance.GetComponent < LaserBehaviour > () as LaserBehaviour;
						TagChecker tagger = Instance.GetComponent<TagChecker>() as TagChecker;
						tagger.barrelPosition = new Vector3(0f, 0f);
						tagger.barrelDirection = oglaser.barrelDirection;
						tagger.PhysicalBehaviour = oglaser.PhysicalBehaviour;
						tagger.LaserColourGradient = oglaser.LaserColourGradient;
						tagger.LayersToHit = oglaser.LayersToHit;
						tagger.LineRenderer = oglaser.LineRenderer;
						tagger.Activated = true;
						oglaser.enabled = false;
					}
                  }
            });*/
         ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Rod"),
                  NameOverride = "Activator",
				  NameToOrderByOverride = "zzzz",
                  DescriptionOverride = "Activates anything it touches (if ON).",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("athumb.png"),
                  AfterSpawn = (Instance) => {
                     Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("activator.png");
                     Instance.FixColliders();
                     Instance.CreateGlowObject(ModAPI.LoadSprite("glow.png"),new Vector3(0f, 0f),new Vector3(0.5f, 0.5f));
					 if(Instance.GetComponent<ActivatorBehaviour>() == null)
					 {
                     Instance.AddComponent(typeof (ActivatorBehaviour));
					 }
                  }
            });
         ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Rod"),
                  NameOverride = "Power Gate",
				  NameToOrderByOverride = "zzzz",
                  DescriptionOverride = "Doesnt let power through unless activated by activation signal.",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("powergateview.png"),
                  AfterSpawn = (Instance) => {
                     Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("pgate.png");
                     Instance.FixColliders();
                     Instance.CreateGlowObject(ModAPI.LoadSprite("glow.png"),new Vector3(-0.115f, 0f),new Vector3(0.5f, 0.5f));
					 if(Instance.GetComponent<PowerGate>() == null)
					 {
                     PowerGate powergate = Instance.AddComponent(typeof (PowerGate)) as PowerGate;
					 }
                  }
            });
         ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Rod"),
                  //item to derive from
                  NameOverride = "Gravity Nullifier",
				  NameToOrderByOverride = "zzzz",
                  //new item name with a suffix to assure it is globally unique
                  DescriptionOverride = "Useful for hovering machines of all sorts.",
                  //new item description
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  //new item category
                  ThumbnailOverride = ModAPI.LoadSprite("nullthumb.png"),
                  //new item thumbnail (relative path)
                  AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                  {
                     Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("nuller.png"); //get the SpriteRenderer and replace its sprite with a custom one
                     Instance.FixColliders();
                     //Instance.AddComponent < PolygonCollider2D > ();
                     Instance.CreateGlowObject(ModAPI.LoadSprite("glow.png"),new Vector3(0f,0f),new Vector3(1f,1f));
                     velstabbh comp = Instance.GetOrAddComponent< velstabbh > ();
                  }
            });
			ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Rod"),
                  NameOverride = "Anchor Part",
				  NameToOrderByOverride = "zzzz",
                  DescriptionOverride = "Experimental technology allows this part to freeze all motion when activated.",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("anchorblockthumb.png"),
                  AfterSpawn = (Instance) => {
                     Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("anchorblock.png");
                     Instance.FixColliders();
					 AnchorBlockBehaviour AnchorBlock = Instance.GetOrAddComponent<AnchorBlockBehaviour>();
					 var light = ModAPI.CreateLight(Instance.transform, Color.green, 5, 2);
					 AnchorBlock.light = light.gameObject;
                  }
            });
			ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Rod"),
                  NameOverride = "Player Part",
				  NameToOrderByOverride = "zzzz",
                  DescriptionOverride = "A part controlled by the player.",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("mousepartt.png"),
                  AfterSpawn = (Instance) => {
                     Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("mousepart.png",1.5f);
                     Instance.FixColliders();
					 Instance.name = "PlayerPart";
					 Rigidbody2D RB = Instance.GetComponent<Rigidbody2D>() as Rigidbody2D;
					 if(Instance.GetComponent<MousePartBehaviour>() == null)
					{
					 Instance.AddComponent(typeof (MousePartBehaviour));
					}
                  }
            });
			ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Defibrillator"),
                  NameOverride = "AND Gate",
				  NameToOrderByOverride = "zzLogicGate_A",
                  DescriptionOverride = "An AND gate.",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("logicgate/and.png"),
                  AfterSpawn = (Instance) => {
					GameObject MainBody = Instance.transform.Find("defib body").gameObject;
					MainBody.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("logicgate/andgate.png");
			        MainBody.FixColliders();
					LogicGateSetup<AndGate>(Instance,ModAPI.LoadSprite("logicgate/g_andgate.png"));
                  }
            });
			ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Defibrillator"),
                  NameOverride = "NOT Gate",
				  NameToOrderByOverride = "zzLogicGate_N",
                  DescriptionOverride = "A NOT gate.",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("logicgate/not.png"),
                  AfterSpawn = (Instance) => {
					GameObject MainBody = Instance.transform.Find("defib body").gameObject;
					MainBody.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("logicgate/notgate.png");
			        MainBody.FixColliders();
					LogicGateSetup<NotGate>(Instance,ModAPI.LoadSprite("logicgate/g_notgate.png"));
					Instance.transform.Find("defib pad 2").gameObject.SetActive(false);
					GameObject slot1 = Instance.transform.Find("defib pad").gameObject;
					slot1.transform.SetParent(MainBody.transform);
					slot1.transform.localPosition = new Vector3(0f,slot1.transform.localPosition.y);
					slot1.transform.SetParent(Instance.transform);
                  }
            });
			ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Defibrillator"),
                  NameOverride = "Counting Gate",
				  NameToOrderByOverride = "zzLogicGate_CG",
                  DescriptionOverride = "A counter. Top input resets.",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("logicgate/count.png"),
                  AfterSpawn = (Instance) => {
					GameObject MainBody = Instance.transform.Find("defib body").gameObject;
					MainBody.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("logicgate/counter.png");
			        MainBody.FixColliders();
					LogicGateSetup<CounterGate>(Instance,ModAPI.LoadSprite("logicgate/g_counter.png"),false);
					GameObject slot2 = Instance.transform.Find("defib pad 2").gameObject;
					GameObject slot1 = Instance.transform.Find("defib pad").gameObject;
					slot2.transform.Find("glow").gameObject.GetComponent<SpriteRenderer>().color = new Color32(255,0,0,255);
					slot1.transform.SetParent(MainBody.transform);
					slot2.transform.SetParent(MainBody.transform);
					slot1.transform.localPosition = new Vector3(0f,-0.4425f + 0.1f);
					slot2.transform.localPosition = new Vector3(0f,0.4425f - 0.1f);
					slot2.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("logicgate/fuck.png");
					slot1.transform.SetParent(Instance.transform);
					slot2.transform.SetParent(Instance.transform);
                  }
            });
			ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Defibrillator"),
                  NameOverride = "OR Gate",
				  NameToOrderByOverride = "zzLogicGate_O",
                  DescriptionOverride = "An OR gate.",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("logicgate/or.png"),
                  AfterSpawn = (Instance) => {
					GameObject MainBody = Instance.transform.Find("defib body").gameObject;
					MainBody.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("logicgate/orgate.png");
			        MainBody.FixColliders();
					LogicGateSetup<OrGate>(Instance,ModAPI.LoadSprite("logicgate/g_orgate.png"));
                  }
            });
			ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Rod"),
                  NameOverride = "Lightblock",
				  NameToOrderByOverride = "zzzz",
                  DescriptionOverride = "Glows when activated. You can put words as colors in the color selection menu.",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("lbt.png"),
                  AfterSpawn = (Instance) => {
					Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("lightblock.png");
			        Instance.FixColliders();
					Instance.CreateGlowObject(ModAPI.LoadSprite("glowwhite.png"),new Vector3(0f, 0f),new Vector3(0.5f,0.5f));
					if(Instance.GetComponent<LightblockBehaviour>() == null)
					{
					 Instance.AddComponent(typeof (LightblockBehaviour));
					}
					Instance.GetComponent<LightblockBehaviour>().lightsprite = ModAPI.CreateLight(Instance.transform, Color.green, 5, 2);
					Instance.GetComponent<LightblockBehaviour>().lightsprite.Brightness = 0;
                  }
            });
			ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Rod"),
                  NameOverride = "Repeater",
				  NameToOrderByOverride = "zzzz",
                  DescriptionOverride = "Metronome but better.",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("repeatert.png"),
                  AfterSpawn = (Instance) => {
					Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("repeater.png");
			        Instance.FixColliders();
					Instance.CreateGlowObject(ModAPI.LoadSprite("glow.png"),new Vector3(-0.075f, 0f),new Vector3(0.5f,0.5f));
					if(Instance.GetComponent<RepeaterBehaviour>() == null)
					{
					 Instance.AddComponent(typeof (RepeaterBehaviour));
					}
					Instance.GetComponent<RepeaterBehaviour>().lightsprite = ModAPI.CreateLight(Instance.transform, Color.green, 5, 2);
					Instance.GetComponent<RepeaterBehaviour>().lightsprite.Brightness = 0;
                  }
            });
			ModAPI.Register(
            new Modification() {
               OriginalItem = ModAPI.FindSpawnable("Defibrillator"),
                  NameOverride = "XOR Gate",
				  NameToOrderByOverride = "zzLogicGate_X",
                  DescriptionOverride = "A XOR gate.",
                  CategoryOverride = ModAPI.FindCategory("Machinery"),
                  ThumbnailOverride = ModAPI.LoadSprite("logicgate/xt.png"),
                  AfterSpawn = (Instance) => {
					GameObject MainBody = Instance.transform.Find("defib body").gameObject;
					MainBody.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("logicgate/xorgate.png");
			        MainBody.FixColliders();
					LogicGateSetup<XorGate>(Instance,ModAPI.LoadSprite("logicgate/g_xorgate.png"));
                  }
            });
			    ModAPI.Register(
			      new Modification() {
			        OriginalItem = ModAPI.FindSpawnable("Detector"), //item to derive from
			          NameOverride = "Omnisensor", //new item name with a suffix to assure it is globally unique
					  NameToOrderByOverride = "zzzz",
			          DescriptionOverride = "A sensor that is way more flexible in certain ways.\nDo note that modes can combine.", //new item description
			          CategoryOverride = ModAPI.FindCategory("Machinery"), //new item category
			          ThumbnailOverride = ModAPI.LoadSprite("omnisensort.png"), //new item thumbnail (relative path)
			          AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
			          {
			            if (Instance.GetComponent < OmnisensorBehaviour > () == null) {
			              Instance.AddComponent(typeof (OmnisensorBehaviour));
			            }
			          }
			      }
			    );
			    ModAPI.Register(
			      new Modification() {
			        OriginalItem = ModAPI.FindSpawnable("Rod"),
					NameToOrderByOverride = "zzzz",
			          NameOverride = "Activation Gate",
			          DescriptionOverride = "Its like a power gate,but enabled by blue activation, turned off by red.",
			          CategoryOverride = ModAPI.FindCategory("Machinery"),
			          ThumbnailOverride = ModAPI.LoadSprite("agt.png",1,false),
			          AfterSpawn = (Instance) => {
			            Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("agate.png");
			            Instance.FixColliders();
						ChangeWeight(Instance,0.75f);
			            Instance.CreateGlowObject( ModAPI.LoadSprite("g_agate.png"), new Vector3(0f, 0f), new Vector3(1f, 1f));
			            Instance.GetOrAddComponent<ActivationGate2>();
			          }
			      });
				//CLUTTER2.cs
				ModAPI.Register(
			      new Modification() {
			        OriginalItem = ModAPI.FindSpawnable("Rod"),
			          NameOverride = "Enviromental Changer",
					  NameToOrderByOverride = "zzzz",
			          DescriptionOverride = "Changes enviroment options when activated and back. Blue wire to create new default template.",
			          CategoryOverride = ModAPI.FindCategory("Machinery"),
			          ThumbnailOverride = ModAPI.LoadSprite("changert.png"),
			          AfterSpawn = (Instance) => {
			            Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("changer.png");
			            Instance.FixColliders();
						ChangeWeight(Instance,1f);
			            Instance.CreateGlowObject( ModAPI.LoadSprite("g_changer.png"), new Vector3(0f, 0f), new Vector3(1f, 1f));
			            if (Instance.GetComponent < EnviromentChanger > () == null) {
			              Instance.AddComponent(typeof (EnviromentChanger));
			            }
			          }
			      });
				  ModAPI.Register(
			      new Modification() {
			        OriginalItem = ModAPI.FindSpawnable("Rod"),
			          NameOverride = "Keycard",
					  NameToOrderByOverride = "zzzz",
			          DescriptionOverride = "Keycard for use with a keycard reader.",
			          CategoryOverride = ModAPI.FindCategory("Machinery"),
			          ThumbnailOverride = ModAPI.LoadSprite("keycardt.png"),
			          AfterSpawn = (Instance) => {
			            Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("keycard.png");
			            Instance.FixColliders();
			            Instance.CreateGlowObject( ModAPI.LoadSprite("c_keycard.png"), new Vector3(0f, 0f), new Vector3(1f, 1f),null,false);
						KeycardBehaviour Keycard = Instance.GetComponent < KeycardBehaviour > () as KeycardBehaviour;
			            if (Keycard == null) {
							//Debug.Log("creatin new");
			              Keycard = Instance.AddComponent(typeof (KeycardBehaviour)) as KeycardBehaviour;
			            }
						//Keycard.beep = ModAPI.LoadSound("");
			          }
			      });
				  ModAPI.Register(
			      new Modification() {
			        OriginalItem = ModAPI.FindSpawnable("Rod"),
			          NameOverride = "Keycard Reader",
					  NameToOrderByOverride = "zzzz",
			          DescriptionOverride = "Sends out a signal when touched by correct keycard.",
			          CategoryOverride = ModAPI.FindCategory("Machinery"),
			          ThumbnailOverride = ModAPI.LoadSprite("readert.png"),
			          AfterSpawn = (Instance) => {
			            Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("keycardreader.png");
			            Instance.FixColliders();
			            Instance.CreateGlowObject( ModAPI.LoadSprite("g_keycardreader.png"), new Vector3(0f, 0f), new Vector3(1f, 1f));
						KeycardReaderBehaviour Keycard = Instance.GetComponent < KeycardReaderBehaviour > () as KeycardReaderBehaviour;
			            if (Keycard == null) {
			              Keycard = Instance.AddComponent(typeof (KeycardReaderBehaviour)) as KeycardReaderBehaviour;
			            }
						Keycard.beep = ModAPI.LoadSound("keycard.mp3");
			          }
			      });
				  ModAPI.Register(
			      new Modification() {
			        OriginalItem = ModAPI.FindSpawnable("Rod"),
			          NameOverride = "Keypad",
					  NameToOrderByOverride = "zzzz",
			          DescriptionOverride = "Enter the password for activation,player parts bring up password menu when touched, or through context menu",
			          CategoryOverride = ModAPI.FindCategory("Machinery"),
			          ThumbnailOverride = ModAPI.LoadSprite("numpadt.png"),
			          AfterSpawn = (Instance) => {
			            Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("numpad.png");
			            Instance.FixColliders();
			            Instance.CreateGlowObject( ModAPI.LoadSprite("g_numpad.png"), new Vector3(0f, 0f), new Vector3(1f, 1f));
						KeypadBehaviour Keycard = Instance.GetComponent < KeypadBehaviour > () as KeypadBehaviour;
			            if (Keycard == null) {
			              Keycard = Instance.AddComponent(typeof (KeypadBehaviour)) as KeypadBehaviour;
			            }
						Keycard.beep = ModAPI.LoadSound("keycard.mp3");
			          }
			      });
				  ModAPI.Register(
			      new Modification() {
			        OriginalItem = ModAPI.FindSpawnable("Rod"),
			          NameOverride = "Destroyer",
					  NameToOrderByOverride = "zzzz",
			          DescriptionOverride = "Destroys anything it touches. USE WITH CAUTION. ESPECIALLY USE WITH CAUTION ON ADVANCED MODE.",
			          CategoryOverride = ModAPI.FindCategory("Machinery"),
			          ThumbnailOverride = ModAPI.LoadSprite("deletort.png"),
			          AfterSpawn = (Instance) => {
			            Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("deletor.png");
			            Instance.FixColliders();
			            Instance.CreateGlowObject( ModAPI.LoadSprite("g_deletor.png"), new Vector3(0f, 0f), new Vector3(1f,1f));
						DestroyerBehaviour Destroyer = Instance.GetOrAddComponent < DestroyerBehaviour > ();
			          }
			      });
				  //GameObject.Instantiate(ModAPI.FindSpawnable("Decimator").Prefab.GetComponent<DecimatorBehaviour>().BlackHole, gameObject.transform.position, Quaternion.identity);
				  ModAPI.Register(
			      new Modification() {
			        OriginalItem = ModAPI.FindSpawnable("Rod"),
			          NameOverride = "Activation Converter",
					  NameToOrderByOverride = "zzzz",
			          DescriptionOverride = "Converts Continuous activation to single activation.",
			          CategoryOverride = ModAPI.FindCategory("Machinery"),
			          ThumbnailOverride = ModAPI.LoadSprite("convertort.png"),
			          AfterSpawn = (Instance) => {
			            Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("convertor.png");
			            Instance.FixColliders();
			            //Instance.CreateGlowObject( ModAPI.LoadSprite("glow.png"), new Vector3(0f, 0f), new Vector3(0.5f,0.5f));
						ActivationConverter Convertor = Instance.GetOrAddComponent < ActivationConverter > ();
			          }
			      });
				  ModAPI.Register(
    new Modification() {
        OriginalItem = ModAPI.FindSpawnable("Defibrillator"),
            NameOverride = "Height Sensor",
			NameToOrderByOverride = "zzzz",
            DescriptionOverride = "Sends a signal when entering specified height range.",
            CategoryOverride = ModAPI.FindCategory("Machinery"),
            ThumbnailOverride = ModAPI.LoadSprite("t_HeightSensor.png"),
            AfterSpawn = (Instance) => {
				GameObject MainBody = Instance.transform.Find("defib body").gameObject;
				ComponentStrip(MainBody);
                MainBody.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("HeightSensor.png");
                MainBody.FixColliders();
                MainBody.CreateGlowObject(ModAPI.LoadSprite("g_HeightSensor.png"), new Vector3(0f, 0f), new Vector3(1f, 1f));
				Instance.transform.Find("defib pad").gameObject.SetActive(false);
				Instance.transform.Find("defib pad 2").gameObject.SetActive(false);
                TextMeshPro Text = MainBody.transform.Find("text").gameObject.GetComponent < TextMeshPro > ();
                Text.enabled = true;
                Text.autoSizeTextContainer = true;
                Text.enableAutoSizing = true;
                Text.transform.localPosition = new Vector3(-0.25f, 0.0725f);
                Text.fontSizeMin = 0;
                Text.alignment = TextAlignmentOptions.Center;
                Text.fontSizeMax = 1;
                //Text.gameObject.transform.SetParent(Instance.transform);
                HeightSensor Sensor = MainBody.GetOrAddComponent < HeightSensor > ();
                Sensor.text = Text;
            }
    });
				  ModAPI.Register(
			      new Modification() {
			        OriginalItem = ModAPI.FindSpawnable("Rod"),
			          NameOverride = "Blood Engine",
					  NameToOrderByOverride = "zzzz",
			          DescriptionOverride = "Converts blood to power. Your loss is our fuel. (Not a famous youtuber quote.)",
			          CategoryOverride = ModAPI.FindCategory("Machinery"),
			          ThumbnailOverride = ModAPI.LoadSprite("t_bloodengine.png"),
			          AfterSpawn = (Instance) => {
			            Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("bloodengine.png");
			            Instance.FixColliders();
			            //Instance.CreateGlowObject( ModAPI.LoadSprite("glow.png"), new Vector3(0f, 0f), new Vector3(0.5f,0.5f));
						Instance.GetOrAddComponent < BloodTransfererEngine > ();
						BloodEngineBehaviour Engine = Instance.GetOrAddComponent < BloodEngineBehaviour > ();
			          }
			      });
				  ModAPI.Register(
			      new Modification() {
			        OriginalItem = ModAPI.FindSpawnable("Rod"),
			          NameOverride = "Light Shrapnel",
					  //NameToOrderByOverride = "zzzz",
			          DescriptionOverride = "Small, sharp jagged metal.",
			          CategoryOverride = ModAPI.FindCategory("Misc."),
			          ThumbnailOverride = ModAPI.LoadSprite("shrapnelt.png"),
			          AfterSpawn = (Instance) => {
						var rnd = new System.Random();
						var bl = rnd.Next(0,3);
						var blg = new Sprite[]{ModAPI.LoadSprite("moretextures/shrapnelA.png"),ModAPI.LoadSprite("moretextures/shrapnelB.png"),ModAPI.LoadSprite("moretextures/shrapnelC.png")};
			            Instance.GetComponent < SpriteRenderer > ().sprite = blg[bl];
			            Instance.FixColliders();
						PhysicalBehaviour pb = Instance.GetComponent<PhysicalBehaviour>();
                        PhysicalProperties prop = ModAPI.FindPhysicalProperties("Metal").ShallowClone();
                        prop.Sharp = true;
						float minx = 0f;
						float maxx = 0.25f;
						bool looseupper = true;
						bool loosedown = true;
                        prop.SharpAxes = new[]
                        {
                            new SharpAxis(Vector2.right, minx,maxx, loosedown, looseupper),
							new SharpAxis(-Vector2.right, minx,maxx, loosedown, looseupper),
							new SharpAxis(Vector2.up, minx,maxx, loosedown, looseupper),
							new SharpAxis(-Vector2.up, minx,maxx, loosedown, looseupper),
                        };
                        pb.Properties = prop;
						ChangeWeight(Instance,0.04f);
			          }
			      });
				  ModAPI.Register(
			      new Modification() {
			        OriginalItem = ModAPI.FindSpawnable("Rod"),
			          NameOverride = "Heavy Shrapnel",
					  //NameToOrderByOverride = "zzzz",
			          DescriptionOverride = "Bigger, less sharp jagged metal.",
			          CategoryOverride = ModAPI.FindCategory("Misc."),
			          ThumbnailOverride = ModAPI.LoadSprite("hshrapnelt.png"),
			          AfterSpawn = (Instance) => {
						var rnd = new System.Random();
						var bl = rnd.Next(0,3);
						var blg = new Sprite[]{ModAPI.LoadSprite("moretextures/hshrapnelA.png"),ModAPI.LoadSprite("moretextures/hshrapnelB.png"),ModAPI.LoadSprite("moretextures/hshrapnelC.png")};
			            Instance.GetComponent < SpriteRenderer > ().sprite = blg[bl];
			            Instance.FixColliders();
						PhysicalBehaviour pb = Instance.GetComponent<PhysicalBehaviour>();
                        PhysicalProperties prop = ModAPI.FindPhysicalProperties("Metal").ShallowClone();
                        prop.Sharp = true;
						float minx = 0f;
						float maxx = 0.1f;
						bool looseupper = true;
						bool loosedown = false;
                        prop.SharpAxes = new[]
                        {
                            new SharpAxis(Vector2.right, minx,maxx, loosedown, looseupper),
							new SharpAxis(-Vector2.right, minx,maxx, loosedown, looseupper),
							new SharpAxis(Vector2.up, minx,maxx, loosedown, looseupper),
							new SharpAxis(-Vector2.up, minx,maxx, loosedown, looseupper),
                        };
                        pb.Properties = prop;
						//Debug.Log(Instance.GetComponent<Rigidbody2D>().mass);
						ChangeWeight(Instance,2f);
			          }
			      });
				  ModAPI.Register(
			      new Modification() {
			        OriginalItem = ModAPI.FindSpawnable("Rod"),
			          NameOverride = "Touchpad",
					  NameToOrderByOverride = "zzzz",
			          DescriptionOverride = "Shock detector, but its more customizable (and compact).",
			          CategoryOverride = ModAPI.FindCategory("Machinery"),
			          ThumbnailOverride = ModAPI.LoadSprite("touchpadt.png"),
			          AfterSpawn = (Instance) => {
						var blg = new Sprite[]{ModAPI.LoadSprite("moretextures/touchpad1.png"),ModAPI.LoadSprite("moretextures/touchpad2.png"),ModAPI.LoadSprite("moretextures/touchpad3.png")};
						SpriteChangerBehaviour Changer = Instance.GetOrAddComponent<SpriteChangerBehaviour>();
						//Debug.Log("possible error start");
						Changer.sprites = blg;
						Changer.Init();
						//Debug.Log("start");
						ChangeWeight(Instance,0.5f);
						TouchpadBehaviour Toucher = Instance.GetOrAddComponent<TouchpadBehaviour>();
						//Debug.Log("end");
			          }
			      });
				  ModAPI.Register(
			      new Modification() {
			        OriginalItem = ModAPI.FindSpawnable("Resistor"),
			          NameOverride = "Lever",
					  NameToOrderByOverride = "zzzz",
			          DescriptionOverride = "Resistor but lever. Sends red signal when deactivating, sends blue signal when activating, and sends green with both.",
			          CategoryOverride = ModAPI.FindCategory("Machinery"),
			          ThumbnailOverride = ModAPI.LoadSprite("leverview.png"),
			          AfterSpawn = (Instance) => {
						  Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("leverbase.png");
						  Instance.FixColliders();
						  GameObject handle = Instance.transform.Find("Handle").gameObject;
						  handle.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("leverlever.png");
						  handle.FixColliders();
						  Debug.Log(handle.transform.localPosition.ToString());
						  handle.transform.localPosition = new Vector3(0f,0.275f);
						  UnityEngine.Object.Destroy(Instance.GetComponent<ResistorBehaviour>());
						  Instance.CreateGlowObject( ModAPI.LoadSprite("moretextures/g_leverg.png"), new Vector3(0f, 0f), new Vector3(1f,1f),extraname: "1");
						  Instance.CreateGlowObject( ModAPI.LoadSprite("moretextures/g_leverr.png"), new Vector3(0f, 0f), new Vector3(1f,1f),extraname: "2");
						  Instance.GetOrAddComponent<LeverBehaviour>();
			          }
			      });
				  ModAPI.Register(
			      new Modification() {
			        OriginalItem = ModAPI.FindSpawnable("Rod"),
			          NameOverride = "Chance Gate",
					  NameToOrderByOverride = "zzzz",
			          DescriptionOverride = "Certain chance for signal to pass. Easy.",
			          CategoryOverride = ModAPI.FindCategory("Machinery"),
			          ThumbnailOverride = ModAPI.LoadSprite("moretextures/chancet.png"),
			          AfterSpawn = (Instance) => {
						  Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("moretextures/chance.png");
						  Instance.FixColliders();
						  Instance.CreateGlowObject( ModAPI.LoadSprite("moretextures/g_chanc_g.png"), new Vector3(0f, 0f), new Vector3(1f,1f),extraname: "1");
						  Instance.CreateGlowObject( ModAPI.LoadSprite("moretextures/g_chance_r.png"), new Vector3(0f, 0f), new Vector3(1f,1f),extraname: "2");
						  Instance.GetOrAddComponent<ChanceGate>();
			          }
			      });
				  ModAPI.Register(
			      new Modification() {
			        OriginalItem = ModAPI.FindSpawnable("Rod"),
			          NameOverride = "Particle Emitter",
					  NameToOrderByOverride = "zzzz",
			          DescriptionOverride = "Makes particles. Once with green,Red makes it loop, and Blue makes it not loop.",
			          CategoryOverride = ModAPI.FindCategory("Machinery"),
			          ThumbnailOverride = ModAPI.LoadSprite("emitter.png"),
			          AfterSpawn = (Instance) => {
						  Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("particler.png");
						  Instance.FixColliders();
						  Instance.GetOrAddComponent<ParticleMaker>();
			          }
			      });
				  ModAPI.Register(
			      new Modification() {
			        OriginalItem = ModAPI.FindSpawnable("Rod"),
			          NameOverride = "pneis",
					  NameToOrderByOverride = "zzzz",
			          DescriptionOverride = "a british favortuerazhsdhzgazudágz",
			          CategoryOverride = ModAPI.FindCategory("amog"),
			          ThumbnailOverride = ModAPI.LoadSprite("pneis.png"),
			          AfterSpawn = (Instance) => {
						  Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("pneis.png");
						  Instance.FixColliders();
						  Instance.GetOrAddComponent<pneis>();
			          }
			      });
				  ModAPI.Register(
			      new Modification() {
			        OriginalItem = ModAPI.FindSpawnable("Rod"),
			          NameOverride = "Inverse Power Gate",
					  NameToOrderByOverride = "zzzz",
			          DescriptionOverride = "Allows activation if its not powered.",
			          CategoryOverride = ModAPI.FindCategory("Machinery"),
			          ThumbnailOverride = ModAPI.LoadSprite("inverset.png"),
			          AfterSpawn = (Instance) => {
						  Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("inversegate.png");
						  Instance.FixColliders();
						  Instance.CreateGlowObject( ModAPI.LoadSprite("moretextures/g_ingate.png"), new Vector3(0f, 0f), new Vector3(1f,1f));
						  Instance.GetOrAddComponent<inversegate>();
			          }
			      });
				  ModAPI.Register(
			      new Modification() {
			        OriginalItem = ModAPI.FindSpawnable("Servo"),
			          NameOverride = "Multiservo",
					  NameToOrderByOverride = "zzzz",
			          DescriptionOverride = "Just like the servo, but with 2 more angles. Use the corresponding activation color to rotate to that.",
			          CategoryOverride = ModAPI.FindCategory("Machinery"),
			          ThumbnailOverride = ModAPI.LoadSprite("moretextures/t_ms.png"),
			          AfterSpawn = (Instance) => {
						  /*Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("inversegate.png");
						  Instance.FixColliders();
						  Instance.CreateGlowObject( ModAPI.LoadSprite("moretextures/g_ingate.png"), new Vector3(0f, 0f), new Vector3(1f,1f));*/
						  UnityEngine.Object.Destroy(Instance.GetComponent<MotorisedHingeBehaviour>());
						  Instance.GetOrAddComponent<MultiservoBehaviour>();
			          }
			      });
				  ModAPI.Register(
			      new Modification() {
			        OriginalItem = ModAPI.FindSpawnable("Rod"),
			          NameOverride = "Portable Powercell",
					  NameToOrderByOverride = "zzzz",
			          DescriptionOverride = "A portable power cell. When activated discharges its power output,and when deactivated it can be recharged.",
			          CategoryOverride = ModAPI.FindCategory("Machinery"),
			          ThumbnailOverride = ModAPI.LoadSprite("moretextures/t_pcell.png"),
			          AfterSpawn = (Instance) => {
						  Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("moretextures/portablecell.png");
						  Instance.FixColliders();
						  Instance.CreateGlowObject( ModAPI.LoadSprite("moretextures/g_ppowercell.png"), new Vector3(0f, 0f), new Vector3(1f,1f));
						  PowerCellBehaviour cell = Instance.GetOrAddComponent<PowerCellBehaviour>();
						  cell.MaxPowerOutput = 40f;cell.ChargeRate = 20f;cell.MaxStoredCharge = 500f;cell.StoredCharge = cell.MaxStoredCharge;
			          }
			      });
				  ModAPI.Register(
			      new Modification() {
			        OriginalItem = ModAPI.FindSpawnable("Rod"),
			          NameOverride = "MK1 Powercell",
					  NameToOrderByOverride = "zzzz",
			          DescriptionOverride = "A MK1 power cell. When activated discharges its power output,and when deactivated it can be recharged.",
			          CategoryOverride = ModAPI.FindCategory("Machinery"),
			          ThumbnailOverride = ModAPI.LoadSprite("moretextures/t_cell1.png"),
			          AfterSpawn = (Instance) => {
						  Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("moretextures/cell1.png");
						  Instance.FixColliders();
						  Instance.CreateGlowObject( ModAPI.LoadSprite("moretextures/g_cell1.png"), new Vector3(0f, 0f), new Vector3(1f,1f));
						  PowerCellBehaviour cell = Instance.GetOrAddComponent<PowerCellBehaviour>();
						  cell.MaxPowerOutput = 80f;cell.ChargeRate = 60f;cell.MaxStoredCharge = 800f;cell.StoredCharge = cell.MaxStoredCharge;
			          }
			      });
				  ModAPI.Register(
			      new Modification() {
			        OriginalItem = ModAPI.FindSpawnable("Rod"),
			          NameOverride = "MK2 Powercell",
					  NameToOrderByOverride = "zzzz",
			          DescriptionOverride = "A MK2 power cell. When activated discharges its power output,and when deactivated it can be recharged.",
			          CategoryOverride = ModAPI.FindCategory("Machinery"),
			          ThumbnailOverride = ModAPI.LoadSprite("moretextures/t_cell2.png"),
			          AfterSpawn = (Instance) => {
						  Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("moretextures/cell2.png");
						  Instance.FixColliders();
						  Instance.CreateGlowObject( ModAPI.LoadSprite("moretextures/g_cell2.png"), new Vector3(0f, 0f), new Vector3(1f,1f));
						  PowerCellBehaviour cell = Instance.GetOrAddComponent<PowerCellBehaviour>();
						  cell.MaxPowerOutput = 140f;cell.ChargeRate = 120f;cell.MaxStoredCharge = 1300f;cell.StoredCharge = cell.MaxStoredCharge;
			          }
			      });
				  ModAPI.Register(
			      new Modification() {
			        OriginalItem = ModAPI.FindSpawnable("Rod"),
			          NameOverride = "MK3 Powercell",
					  NameToOrderByOverride = "zzzz",
			          DescriptionOverride = "A MK3 power cell. When activated discharges its power output,and when deactivated it can be recharged.",
			          CategoryOverride = ModAPI.FindCategory("Machinery"),
			          ThumbnailOverride = ModAPI.LoadSprite("moretextures/t_cell3.png"),
			          AfterSpawn = (Instance) => {
						  Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("moretextures/cell3.png");
						  Instance.FixColliders();
						  Instance.CreateGlowObject( ModAPI.LoadSprite("moretextures/g_cell3.png"), new Vector3(0f, 0f), new Vector3(1f,1f));
						  PowerCellBehaviour cell = Instance.GetOrAddComponent<PowerCellBehaviour>();
						  cell.MaxPowerOutput = 200f;cell.ChargeRate = 160f;cell.MaxStoredCharge = 2000f;cell.StoredCharge = cell.MaxStoredCharge;
			          }
			      });
				  ModAPI.Register(
			      new Modification() {
			        OriginalItem = ModAPI.FindSpawnable("Rod"),
			          NameOverride = "Unrestricted Powercell",
					  NameToOrderByOverride = "zzzz",
			          DescriptionOverride = "Basically just a creative powercell. You know the drill.",
			          CategoryOverride = ModAPI.FindCategory("Machinery"),
			          ThumbnailOverride = ModAPI.LoadSprite("moretextures/t_ccell.png"),
			          AfterSpawn = (Instance) => {
						  Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("moretextures/creativecell.png");
						  Instance.FixColliders();
						  Instance.CreateGlowObject( ModAPI.LoadSprite("moretextures/g_ppowercell.png"), new Vector3(0f, 0f), new Vector3(1f,1f));
						  PowerCellBehaviour cell = Instance.GetOrAddComponent<PowerCellBehaviour>();
						  Instance.GetOrAddComponent<CreativeCell>();
			          }
			      });
				  /*ModAPI.Register(
			      new Modification() {
			        OriginalItem = ModAPI.FindSpawnable("Rod"),
			          NameOverride = "Trigger Detector",
			          DescriptionOverride = "Detects player parts,etc. in an area.",
			          CategoryOverride = ModAPI.FindCategory("Machinery"),
			          ThumbnailOverride = ModAPI.LoadSprite("placeholder.png"),
			          AfterSpawn = (Instance) => {
			            Instance.GetComponent < SpriteRenderer > ().sprite = ModAPI.LoadSprite("placeholder.png");
			            Instance.FixColliders();
						Instance.layer = 10;
			            //Instance.CreateGlowObject( ModAPI.LoadSprite("glow.png"), new Vector3(0f, 0f), new Vector3(0.5f,0.5f));
						TriggerSensor Sensor = Instance.GetOrAddComponent < TriggerSensor > ();
						BoxCollider2D Trigger = Instance.AddComponent < BoxCollider2D > ();
						Trigger.isTrigger = true;
						Sensor.Trigger = Trigger;
						Debug.Log(Trigger.ToString());
						Debug.Log(Sensor.Trigger.ToString());
			          }
			      });*/
      }
   }
}